@@ -5,8 +5,8 @@
  *	toolkit. This allows cursors to be shared between widgets and also
  *	avoids round-trips to the X server.
  *
- * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright © 1990-1994 The Regents of the University of California.
+ * Copyright © 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -14,6 +14,10 @@
 
 #include "tkInt.h"
 
+#ifdef _WIN32
+#include "tkWinInt.h"
+#endif
+
 /*
  * A TkCursor structure exists for each cursor that is currently active. Each
  * structure is indexed with two hash tables defined below. One of the tables
@@ -23,8 +27,8 @@
  */
 
 typedef struct {
-    CONST char *source;		/* Cursor bits. */
-    CONST char *mask;		/* Mask bits. */
+    const char *source;		/* Cursor bits. */
+    const char *mask;		/* Mask bits. */
     int width, height;		/* Dimensions of cursor (and data and
 				 * mask). */
     int xHot, yHot;		/* Location of cursor hot-spot. */
@@ -40,9 +44,10 @@ static void		CursorInit(TkDisplay *dispPtr);
 static void		DupCursorObjProc(Tcl_Obj *srcObjPtr,
 			    Tcl_Obj *dupObjPtr);
 static void		FreeCursor(TkCursor *cursorPtr);
+static void		FreeCursorObj(Tcl_Obj *objPtr);
 static void		FreeCursorObjProc(Tcl_Obj *objPtr);
-static TkCursor *	TkcGetCursor(Tcl_Interp *interp,
-			    Tk_Window tkwin, CONST char *name);
+static TkCursor *	TkcGetCursor(Tcl_Interp *interp, Tk_Window tkwin,
+			    const char *name);
 static TkCursor *	GetCursorFromObj(Tk_Window tkwin, Tcl_Obj *objPtr);
 static void		InitCursorObj(Tcl_Obj *objPtr);
 
@@ -54,12 +59,14 @@ static void		InitCursorObj(Tcl_Obj *objPtr);
  * option is set.
  */
 
-Tcl_ObjType tkCursorObjType = {
-    "cursor",			/* name */
+const TkObjType tkCursorObjType = {
+    {"cursor",			/* name */
     FreeCursorObjProc,		/* freeIntRepProc */
     DupCursorObjProc,		/* dupIntRepProc */
     NULL,			/* updateStringProc */
-    NULL			/* setFromAnyProc */
+    NULL,			/* setFromAnyProc */
+    TCL_OBJTYPE_V1(TkLengthOne)},
+    0
 };
 
 /*
@@ -96,10 +103,10 @@ Tk_AllocCursorFromObj(
 {
     TkCursor *cursorPtr;
 
-    if (objPtr->typePtr != &tkCursorObjType) {
+    if (objPtr->typePtr != &tkCursorObjType.objType) {
 	InitCursorObj(objPtr);
     }
-    cursorPtr = (TkCursor *) objPtr->internalRep.twoPtrValue.ptr1;
+    cursorPtr = (TkCursor *)objPtr->internalRep.twoPtrValue.ptr1;
 
     /*
      * If the object currently points to a TkCursor, see if it's the one we
@@ -113,7 +120,7 @@ Tk_AllocCursorFromObj(
 	     * longer in use. Clear the reference.
 	     */
 
-	    FreeCursorObjProc(objPtr);
+	    FreeCursorObj(objPtr);
 	    cursorPtr = NULL;
 	} else if (Tk_Display(tkwin) == cursorPtr->display) {
 	    cursorPtr->resourceRefCount++;
@@ -128,15 +135,15 @@ Tk_AllocCursorFromObj(
      */
 
     if (cursorPtr != NULL) {
-	TkCursor *firstCursorPtr = (TkCursor *)
-		Tcl_GetHashValue(cursorPtr->hashPtr);
-	FreeCursorObjProc(objPtr);
+	TkCursor *firstCursorPtr = (TkCursor *)Tcl_GetHashValue(cursorPtr->hashPtr);
+
+	FreeCursorObj(objPtr);
 	for (cursorPtr = firstCursorPtr;  cursorPtr != NULL;
 		cursorPtr = cursorPtr->nextPtr) {
 	    if (Tk_Display(tkwin) == cursorPtr->display) {
 		cursorPtr->resourceRefCount++;
 		cursorPtr->objRefCount++;
-		objPtr->internalRep.twoPtrValue.ptr1 = (void *) cursorPtr;
+		objPtr->internalRep.twoPtrValue.ptr1 = cursorPtr;
 		return cursorPtr->cursor;
 	    }
 	}
@@ -147,9 +154,9 @@ Tk_AllocCursorFromObj(
      */
 
     cursorPtr = TkcGetCursor(interp, tkwin, Tcl_GetString(objPtr));
-    objPtr->internalRep.twoPtrValue.ptr1 = (void *) cursorPtr;
+    objPtr->internalRep.twoPtrValue.ptr1 = cursorPtr;
     if (cursorPtr == NULL) {
-	return None;
+	return NULL;
     }
     cursorPtr->objRefCount++;
     return cursorPtr->cursor;
@@ -183,12 +190,13 @@ Tk_Cursor
 Tk_GetCursor(
     Tcl_Interp *interp,		/* Interpreter to use for error reporting. */
     Tk_Window tkwin,		/* Window in which cursor will be used. */
-    Tk_Uid string)		/* Description of cursor. See manual entry for
+    const char *string)		/* Description of cursor. See manual entry for
 				 * details on legal syntax. */
 {
     TkCursor *cursorPtr = TkcGetCursor(interp, tkwin, string);
+
     if (cursorPtr == NULL) {
-	return None;
+	return NULL;
     }
     return cursorPtr->cursor;
 }
@@ -223,11 +231,11 @@ static TkCursor *
 TkcGetCursor(
     Tcl_Interp *interp,		/* Interpreter to use for error reporting. */
     Tk_Window tkwin,		/* Window in which cursor will be used. */
-    CONST char *string)		/* Description of cursor. See manual entry for
+    const char *string)		/* Description of cursor. See manual entry for
 				 * details on legal syntax. */
 {
     Tcl_HashEntry *nameHashPtr;
-    register TkCursor *cursorPtr;
+    TkCursor *cursorPtr;
     TkCursor *existingCursorPtr = NULL;
     int isNew;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
@@ -237,9 +245,9 @@ TkcGetCursor(
     }
 
     nameHashPtr = Tcl_CreateHashEntry(&dispPtr->cursorNameTable,
-            string, &isNew);
+	    string, &isNew);
     if (!isNew) {
-	existingCursorPtr = (TkCursor *) Tcl_GetHashValue(nameHashPtr);
+	existingCursorPtr = (TkCursor *)Tcl_GetHashValue(nameHashPtr);
 	for (cursorPtr = existingCursorPtr; cursorPtr != NULL;
 		cursorPtr = cursorPtr->nextPtr) {
 	    if (Tk_Display(tkwin) == cursorPtr->display) {
@@ -271,7 +279,7 @@ TkcGetCursor(
     cursorPtr->hashPtr = nameHashPtr;
     cursorPtr->nextPtr = existingCursorPtr;
     cursorPtr->idHashPtr = Tcl_CreateHashEntry(&dispPtr->cursorIdTable,
-            (char *) cursorPtr->cursor, &isNew);
+	    (char *) cursorPtr->cursor, &isNew);
     if (!isNew) {
 	Tcl_Panic("cursor already registered in Tk_GetCursor");
     }
@@ -309,16 +317,16 @@ Tk_Cursor
 Tk_GetCursorFromData(
     Tcl_Interp *interp,		/* Interpreter to use for error reporting. */
     Tk_Window tkwin,		/* Window in which cursor will be used. */
-    CONST char *source,		/* Bitmap data for cursor shape. */
-    CONST char *mask,		/* Bitmap data for cursor mask. */
+    const char *source,		/* Bitmap data for cursor shape. */
+    const char *mask,		/* Bitmap data for cursor mask. */
     int width, int height,	/* Dimensions of cursor. */
     int xHot, int yHot,		/* Location of hot-spot in cursor. */
-    Tk_Uid fg,			/* Foreground color for cursor. */
-    Tk_Uid bg)			/* Background color for cursor. */
+    const char *fg,			/* Foreground color for cursor. */
+    const char *bg)			/* Background color for cursor. */
 {
     DataKey dataKey;
     Tcl_HashEntry *dataHashPtr;
-    register TkCursor *cursorPtr;
+    TkCursor *cursorPtr;
     int isNew;
     XColor fgColor, bgColor;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
@@ -333,13 +341,13 @@ Tk_GetCursorFromData(
     dataKey.height = height;
     dataKey.xHot = xHot;
     dataKey.yHot = yHot;
-    dataKey.fg = fg;
-    dataKey.bg = bg;
+    dataKey.fg = Tk_GetUid(fg);
+    dataKey.bg = Tk_GetUid(bg);
     dataKey.display = Tk_Display(tkwin);
     dataHashPtr = Tcl_CreateHashEntry(&dispPtr->cursorDataTable,
-            (char *) &dataKey, &isNew);
+	    (char *) &dataKey, &isNew);
     if (!isNew) {
-	cursorPtr = (TkCursor *) Tcl_GetHashValue(dataHashPtr);
+	cursorPtr = (TkCursor *)Tcl_GetHashValue(dataHashPtr);
 	cursorPtr->resourceRefCount++;
 	return cursorPtr->cursor;
     }
@@ -350,11 +358,15 @@ Tk_GetCursorFromData(
      */
 
     if (TkParseColor(dataKey.display, Tk_Colormap(tkwin), fg, &fgColor) == 0) {
-	Tcl_AppendResult(interp, "invalid color name \"", fg, "\"", NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"invalid color name \"%s\"", fg));
+	Tcl_SetErrorCode(interp, "TK", "VALUE", "CURSOR", "COLOR", NULL);
 	goto error;
     }
     if (TkParseColor(dataKey.display, Tk_Colormap(tkwin), bg, &bgColor) == 0) {
-	Tcl_AppendResult(interp, "invalid color name \"", bg, "\"", NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"invalid color name \"%s\"", bg));
+	Tcl_SetErrorCode(interp, "TK", "VALUE", "CURSOR", "COLOR", NULL);
 	goto error;
     }
 
@@ -370,7 +382,7 @@ Tk_GetCursorFromData(
     cursorPtr->hashPtr = dataHashPtr;
     cursorPtr->objRefCount = 0;
     cursorPtr->idHashPtr = Tcl_CreateHashEntry(&dispPtr->cursorIdTable,
-            (char *) cursorPtr->cursor, &isNew);
+	    (char *) cursorPtr->cursor, &isNew);
     cursorPtr->nextPtr = NULL;
 
     if (!isNew) {
@@ -382,7 +394,7 @@ Tk_GetCursorFromData(
 
   error:
     Tcl_DeleteHashEntry(dataHashPtr);
-    return None;
+    return NULL;
 }
 
 /*
@@ -405,7 +417,7 @@ Tk_GetCursorFromData(
  *--------------------------------------------------------------
  */
 
-CONST char *
+const char *
 Tk_NameOfCursor(
     Display *display,		/* Display for which cursor was allocated. */
     Tk_Cursor cursor)		/* Identifier for cursor whose name is
@@ -419,14 +431,14 @@ Tk_NameOfCursor(
 
     if (!dispPtr->cursorInit) {
     printid:
-	sprintf(dispPtr->cursorString, "cursor id %p", cursor);
+	snprintf(dispPtr->cursorString, sizeof(dispPtr->cursorString), "cursor id 0x%" TCL_Z_MODIFIER "x", PTR2INT(cursor));
 	return dispPtr->cursorString;
     }
-    idHashPtr = Tcl_FindHashEntry(&dispPtr->cursorIdTable, (char *) cursor);
+    idHashPtr = Tcl_FindHashEntry(&dispPtr->cursorIdTable, cursor);
     if (idHashPtr == NULL) {
 	goto printid;
     }
-    cursorPtr = (TkCursor *) Tcl_GetHashValue(idHashPtr);
+    cursorPtr = (TkCursor *)Tcl_GetHashValue(idHashPtr);
     if (cursorPtr->otherTable != &dispPtr->cursorNameTable) {
 	goto printid;
     }
@@ -457,13 +469,12 @@ FreeCursor(
 {
     TkCursor *prevPtr;
 
-    cursorPtr->resourceRefCount--;
-    if (cursorPtr->resourceRefCount > 0) {
+    if (cursorPtr->resourceRefCount-- > 1) {
 	return;
     }
 
     Tcl_DeleteHashEntry(cursorPtr->idHashPtr);
-    prevPtr = (TkCursor *) Tcl_GetHashValue(cursorPtr->hashPtr);
+    prevPtr = (TkCursor *)Tcl_GetHashValue(cursorPtr->hashPtr);
     if (prevPtr == cursorPtr) {
 	if (cursorPtr->nextPtr == NULL) {
 	    Tcl_DeleteHashEntry(cursorPtr->hashPtr);
@@ -478,7 +489,7 @@ FreeCursor(
     }
     TkpFreeCursor(cursorPtr);
     if (cursorPtr->objRefCount == 0) {
-	ckfree((char *) cursorPtr);
+	ckfree(cursorPtr);
     }
 }
 
@@ -512,11 +523,11 @@ Tk_FreeCursor(
 	Tcl_Panic("Tk_FreeCursor called before Tk_GetCursor");
     }
 
-    idHashPtr = Tcl_FindHashEntry(&dispPtr->cursorIdTable, (char *) cursor);
+    idHashPtr = Tcl_FindHashEntry(&dispPtr->cursorIdTable, cursor);
     if (idHashPtr == NULL) {
 	Tcl_Panic("Tk_FreeCursor received unknown cursor argument");
     }
-    FreeCursor((TkCursor *) Tcl_GetHashValue(idHashPtr));
+    FreeCursor((TkCursor *)Tcl_GetHashValue(idHashPtr));
 }
 
 /*
@@ -547,13 +558,13 @@ Tk_FreeCursorFromObj(
     Tcl_Obj *objPtr)		/* The Tcl_Obj * to be freed. */
 {
     FreeCursor(GetCursorFromObj(tkwin, objPtr));
-    FreeCursorObjProc(objPtr);
+    FreeCursorObj(objPtr);
 }
 
 /*
  *---------------------------------------------------------------------------
  *
- * FreeCursorFromObjProc --
+ * FreeCursorObjProc, FreeCursorObj --
  *
  *	This proc is called to release an object reference to a cursor.
  *	Called when the object's internal rep is released or when the cached
@@ -573,13 +584,20 @@ static void
 FreeCursorObjProc(
     Tcl_Obj *objPtr)		/* The object we are releasing. */
 {
-    TkCursor *cursorPtr = (TkCursor *) objPtr->internalRep.twoPtrValue.ptr1;
+    FreeCursorObj(objPtr);
+    objPtr->typePtr = NULL;
+}
+
+static void
+FreeCursorObj(
+    Tcl_Obj *objPtr)		/* The object we are releasing. */
+{
+    TkCursor *cursorPtr = (TkCursor *)objPtr->internalRep.twoPtrValue.ptr1;
 
     if (cursorPtr != NULL) {
-	cursorPtr->objRefCount--;
-	if ((cursorPtr->objRefCount == 0)
+	if ((cursorPtr->objRefCount-- <= 1)
 		&& (cursorPtr->resourceRefCount == 0)) {
-	    ckfree((char *) cursorPtr);
+	    ckfree(cursorPtr);
 	}
 	objPtr->internalRep.twoPtrValue.ptr1 = NULL;
     }
@@ -608,10 +626,10 @@ DupCursorObjProc(
     Tcl_Obj *srcObjPtr,		/* The object we are copying from. */
     Tcl_Obj *dupObjPtr)		/* The object we are copying to. */
 {
-    TkCursor *cursorPtr = (TkCursor *) srcObjPtr->internalRep.twoPtrValue.ptr1;
+    TkCursor *cursorPtr = (TkCursor *)srcObjPtr->internalRep.twoPtrValue.ptr1;
 
     dupObjPtr->typePtr = srcObjPtr->typePtr;
-    dupObjPtr->internalRep.twoPtrValue.ptr1 = (void *) cursorPtr;
+    dupObjPtr->internalRep.twoPtrValue.ptr1 = cursorPtr;
 
     if (cursorPtr != NULL) {
 	cursorPtr->objRefCount++;
@@ -682,7 +700,7 @@ GetCursorFromObj(
     Tcl_HashEntry *hashPtr;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
-    if (objPtr->typePtr != &tkCursorObjType) {
+    if (objPtr->typePtr != &tkCursorObjType.objType) {
 	InitCursorObj(objPtr);
     }
 
@@ -693,7 +711,7 @@ GetCursorFromObj(
      * cached is the one that is needed.
      */
 
-    cursorPtr = (TkCursor *) objPtr->internalRep.twoPtrValue.ptr1;
+    cursorPtr = (TkCursor *)objPtr->internalRep.twoPtrValue.ptr1;
     if ((cursorPtr != NULL) && (Tk_Display(tkwin) == cursorPtr->display)) {
 	return cursorPtr;
     }
@@ -708,11 +726,11 @@ GetCursorFromObj(
     if (hashPtr == NULL) {
 	goto error;
     }
-    for (cursorPtr = (TkCursor *) Tcl_GetHashValue(hashPtr);
+    for (cursorPtr = (TkCursor *)Tcl_GetHashValue(hashPtr);
 	    cursorPtr != NULL; cursorPtr = cursorPtr->nextPtr) {
 	if (Tk_Display(tkwin) == cursorPtr->display) {
-	    FreeCursorObjProc(objPtr);
-	    objPtr->internalRep.twoPtrValue.ptr1 = (void *) cursorPtr;
+	    FreeCursorObj(objPtr);
+	    objPtr->internalRep.twoPtrValue.ptr1 = cursorPtr;
 	    cursorPtr->objRefCount++;
 	    return cursorPtr;
 	}
@@ -757,9 +775,9 @@ InitCursorObj(
     Tcl_GetString(objPtr);
     typePtr = objPtr->typePtr;
     if ((typePtr != NULL) && (typePtr->freeIntRepProc != NULL)) {
-	(*typePtr->freeIntRepProc)(objPtr);
+	typePtr->freeIntRepProc(objPtr);
     }
-    objPtr->typePtr = &tkCursorObjType;
+    objPtr->typePtr = &tkCursorObjType.objType;
     objPtr->internalRep.twoPtrValue.ptr1 = NULL;
 }
 
@@ -830,7 +848,7 @@ Tcl_Obj *
 TkDebugCursor(
     Tk_Window tkwin,		/* The window in which the cursor will be used
 				 * (not currently used). */
-    char *name)			/* Name of the desired color. */
+    const char *name)		/* Name of the desired color. */
 {
     TkCursor *cursorPtr;
     Tcl_HashEntry *hashPtr;
@@ -843,16 +861,16 @@ TkDebugCursor(
     resultPtr = Tcl_NewObj();
     hashPtr = Tcl_FindHashEntry(&dispPtr->cursorNameTable, name);
     if (hashPtr != NULL) {
-	cursorPtr = (TkCursor *) Tcl_GetHashValue(hashPtr);
+	cursorPtr = (TkCursor *)Tcl_GetHashValue(hashPtr);
 	if (cursorPtr == NULL) {
 	    Tcl_Panic("TkDebugCursor found empty hash table entry");
 	}
 	for ( ; (cursorPtr != NULL); cursorPtr = cursorPtr->nextPtr) {
 	    objPtr = Tcl_NewObj();
 	    Tcl_ListObjAppendElement(NULL, objPtr,
-		    Tcl_NewIntObj(cursorPtr->resourceRefCount));
+		    Tcl_NewWideIntObj(cursorPtr->resourceRefCount));
 	    Tcl_ListObjAppendElement(NULL, objPtr,
-		    Tcl_NewIntObj(cursorPtr->objRefCount));
+		    Tcl_NewWideIntObj(cursorPtr->objRefCount));
 	    Tcl_ListObjAppendElement(NULL, resultPtr, objPtr);
 	}
     }
