@@ -5,7 +5,7 @@
  *	parts of the font package. This information is not visible outside of
  *	the font package.
  *
- * Copyright (c) 1996-1997 Sun Microsystems, Inc.
+ * Copyright © 1996-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -14,6 +14,10 @@
 #ifndef _TKFONT
 #define _TKFONT
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * The following structure keeps track of the attributes of a font. It can be
  * used to keep track of either the desired attributes or the actual
@@ -85,7 +89,7 @@ typedef struct TkFont {
      * Fields used and maintained exclusively by generic code.
      */
 
-    int resourceRefCount;	/* Number of active uses of this font (each
+    Tcl_Size resourceRefCount;	/* Number of active uses of this font (each
 				 * active use corresponds to a call to
 				 * Tk_AllocFontFromTable or Tk_GetFont). If
 				 * this count is 0, then this TkFont structure
@@ -95,7 +99,7 @@ typedef struct TkFont {
 				 * The structure is freed when
 				 * resourceRefCount and objRefCount are both
 				 * 0. */
-    int objRefCount;		/* The number of Tcl objects that reference
+    Tcl_Size objRefCount;		/* The number of Tcl objects that reference
 				 * this structure. */
     Tcl_HashEntry *cacheHashPtr;/* Entry in font cache for this structure,
 				 * used when deleting it. */
@@ -221,4 +225,8 @@ MODULE_SCOPE void	TkpGetFontFamilies(Tcl_Interp *interp,
 			    Tk_Window tkwin);
 MODULE_SCOPE TkFont *	TkpGetNativeFont(Tk_Window tkwin, const char *name);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif	/* _TKFONT */
