@@ -39,8 +39,6 @@ typedef struct {
      */
 
     Tcl_Obj *stringObj;		/* String displayed in message. */
-    Tcl_Size numChars;		/* Number of characters in string, not
-				 * including terminating NULL. */
     Tcl_Obj *textVarNameObj;	/* Name of variable or NULL.
 				 * If non-NULL, message displays the contents
 				 * of this variable. */
@@ -473,10 +471,11 @@ ConfigureMessage(
 	    Tcl_SetVar2(interp, Tcl_GetString(msgPtr->textVarNameObj), NULL, Tcl_GetString(msgPtr->stringObj),
 		    TCL_GLOBAL_ONLY);
 	} else {
-	    Tcl_Obj *newObj = Tcl_NewStringObj(value, TCL_INDEX_NONE);
-	    Tcl_IncrRefCount(newObj);
-	    Tcl_DecrRefCount(msgPtr->stringObj);
-	    msgPtr->stringObj = newObj;
+	    if (msgPtr->stringObj != NULL) {
+		Tcl_DecrRefCount(msgPtr->stringObj);
+	    }
+	    msgPtr->stringObj = Tcl_NewStringObj(value, TCL_INDEX_NONE);
+	    Tcl_IncrRefCount(msgPtr->stringObj);
 	}
 	Tcl_TraceVar2(interp, Tcl_GetString(msgPtr->textVarNameObj), NULL,
 		TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
@@ -489,8 +488,6 @@ ConfigureMessage(
      * be specified to Tk_ConfigureWidget.
      */
 
-    msgPtr->numChars = Tcl_NumUtfChars(Tcl_GetString(msgPtr->stringObj), TCL_INDEX_NONE);
-
     Tk_GetPixelsFromObj(NULL, msgPtr->tkwin, msgPtr->widthObj, &width);
     if (width < 0) {
 	if (msgPtr->widthObj) {
@@ -613,11 +610,10 @@ ComputeMessageGeometry(
     int aspect, lowerBound, upperBound, inset;
     int borderWidth, highlightWidth, padX, padY;
     Tk_FontMetrics fm;
+    Tcl_Size numChars;
 
     Tk_FreeTextLayout(msgPtr->textLayout);
 
-    Tk_GetPixelsFromObj(NULL, msgPtr->tkwin, msgPtr->borderWidthObj, &borderWidth);
-    Tk_GetPixelsFromObj(NULL, msgPtr->tkwin, msgPtr->highlightWidthObj, &highlightWidth);
     Tk_GetFontMetrics(msgPtr->tkfont, &fm);
     if (msgPtr->padXObj) {
 	Tk_GetPixelsFromObj(NULL, msgPtr->tkwin, msgPtr->padXObj, &padX);
@@ -630,6 +626,8 @@ ComputeMessageGeometry(
 	padY = fm.ascent / 4;
     }
 
+    Tk_GetPixelsFromObj(NULL, msgPtr->tkwin, msgPtr->borderWidthObj, &borderWidth);
+    Tk_GetPixelsFromObj(NULL, msgPtr->tkwin, msgPtr->highlightWidthObj, &highlightWidth);
     inset = borderWidth + highlightWidth;
 
     /*
@@ -659,9 +657,10 @@ ComputeMessageGeometry(
 	inc = width/2;
     }
 
+    numChars = Tcl_GetCharLength(msgPtr->stringObj);
     for ( ; ; inc /= 2) {
 	msgPtr->textLayout = Tk_ComputeTextLayout(msgPtr->tkfont,
-		Tcl_GetString(msgPtr->stringObj), msgPtr->numChars, width, msgPtr->justify,
+		Tcl_GetString(msgPtr->stringObj), numChars, width, msgPtr->justify,
 		0, &thisWidth, &thisHeight);
 	maxWidth = thisWidth + 2 * (inset + padX);
 	height = thisHeight + 2 * (inset + padY);
@@ -936,11 +935,11 @@ MessageTextVarProc(
     if (value == NULL) {
 	value = "";
     }
-    Tcl_Obj *newObj = Tcl_NewStringObj(value, TCL_INDEX_NONE);
-    Tcl_IncrRefCount(newObj);
-    Tcl_DecrRefCount(msgPtr->stringObj);
-    msgPtr->numChars = Tcl_NumUtfChars(value, TCL_INDEX_NONE);
-    msgPtr->stringObj = newObj;
+    if (msgPtr->stringObj != NULL) {
+	Tcl_DecrRefCount(msgPtr->stringObj);
+    }
+    msgPtr->stringObj = Tcl_NewStringObj(value, TCL_INDEX_NONE);
+    Tcl_IncrRefCount(msgPtr->stringObj);
     ComputeMessageGeometry(msgPtr);
 
     if ((msgPtr->tkwin != NULL) && Tk_IsMapped(msgPtr->tkwin)
