@@ -4,8 +4,8 @@
  *	This module provides functions that manipulate indices for text
  *	widgets.
  *
- * Copyright (c) 1992-1994 The Regents of the University of California.
- * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright © 1992-1994 The Regents of the University of California.
+ * Copyright © 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -45,8 +45,8 @@ static int		GetIndex(Tcl_Interp *interp, TkSharedText *sharedPtr,
 			    TkText *textPtr, const char *string,
 			    TkTextIndex *indexPtr, int *canCachePtr);
 static int              IndexCountBytesOrdered(const TkText *textPtr,
-                            const TkTextIndex *indexPtr1,
-                            const TkTextIndex *indexPtr2);
+			    const TkTextIndex *indexPtr1,
+			    const TkTextIndex *indexPtr2);
 
 /*
  * The "textindex" Tcl_Obj definition:
@@ -68,19 +68,21 @@ static void		UpdateStringOfTextIndex(Tcl_Obj *objPtr);
 #define SET_TEXTINDEX(objPtr, indexPtr) \
 	((objPtr)->internalRep.twoPtrValue.ptr1 = (void *)(indexPtr))
 #define SET_INDEXEPOCH(objPtr, epoch) \
-	((objPtr)->internalRep.twoPtrValue.ptr2 = INT2PTR(epoch))
+	((objPtr)->internalRep.twoPtrValue.ptr2 = (void *) (size_t) (epoch))
 
 /*
  * Define the 'textindex' object type, which Tk uses to represent indices in
  * text widgets internally.
  */
 
-const Tcl_ObjType tkTextIndexType = {
-    "textindex",		/* name */
+const TkObjType tkTextIndexType = {
+    {"textindex",		/* name */
     FreeTextIndexInternalRep,	/* freeIntRepProc */
     DupTextIndexInternalRep,	/* dupIntRepProc */
     NULL,			/* updateStringProc */
-    NULL			/* setFromAnyProc */
+    NULL,			/* setFromAnyProc */
+    TCL_OBJTYPE_V0},
+    0
 };
 
 static void
@@ -108,7 +110,7 @@ DupTextIndexInternalRep(
     Tcl_Obj *srcPtr,		/* TextIndex obj with internal rep to copy. */
     Tcl_Obj *copyPtr)		/* TextIndex obj with internal rep to set. */
 {
-    int epoch;
+    Tcl_Size epoch;
     TkTextIndex *dupIndexPtr, *indexPtr;
 
     dupIndexPtr = (TkTextIndex *)ckalloc(sizeof(TkTextIndex));
@@ -124,7 +126,7 @@ DupTextIndexInternalRep(
     }
     SET_TEXTINDEX(copyPtr, dupIndexPtr);
     SET_INDEXEPOCH(copyPtr, epoch);
-    copyPtr->typePtr = &tkTextIndexType;
+    copyPtr->typePtr = &tkTextIndexType.objType;
 }
 
 /*
@@ -186,7 +188,7 @@ MakeObjIndex(
     indexPtr->linePtr = origPtr->linePtr;
     indexPtr->byteIndex = origPtr->byteIndex;
     SET_TEXTINDEX(objPtr, indexPtr);
-    objPtr->typePtr = &tkTextIndexType;
+    objPtr->typePtr = &tkTextIndexType.objType;
     indexPtr->textPtr = textPtr;
 
     if (textPtr != NULL) {
@@ -209,8 +211,8 @@ TkTextGetIndexFromObj(
     TkTextIndex *indexPtr = NULL;
     int cache;
 
-    if (objPtr->typePtr == &tkTextIndexType) {
-	int epoch;
+    if (objPtr->typePtr == &tkTextIndexType.objType) {
+	Tcl_Size epoch;
 
 	indexPtr = GET_TEXTINDEX(objPtr);
 	epoch = GET_INDEXEPOCH(objPtr);
@@ -385,7 +387,7 @@ TkTextMakeByteIndex(
     const TkText *textPtr,
     int lineIndex,		/* Index of desired line (0 means first line
 				 * of text). */
-    int byteIndex,		/* Byte index of desired character. */
+    Tcl_Size byteIndex,		/* Byte index of desired character. */
     TkTextIndex *indexPtr)	/* Structure to fill in. */
 {
     TkTextSegment *segPtr;
@@ -429,7 +431,7 @@ TkTextMakeByteIndex(
 	    indexPtr->byteIndex = index - sizeof(char);
 	    break;
 	}
-	if (index + segPtr->size > byteIndex) {
+	if (index + (int)segPtr->size > byteIndex) {
 	    indexPtr->byteIndex = byteIndex;
 	    if ((byteIndex > index) && (segPtr->typePtr == &tkTextCharType)) {
 		/*
@@ -440,8 +442,8 @@ TkTextMakeByteIndex(
 		 */
 
 		start = segPtr->body.chars + (byteIndex - index);
-		p = TkUtfPrev(start, segPtr->body.chars);
-		p += TkUtfToUniChar(p, &ch);
+		p = Tcl_UtfPrev(start, segPtr->body.chars);
+		p += Tcl_UtfToUniChar(p, &ch);
 		indexPtr->byteIndex += p - start;
 	    }
 	    break;
@@ -535,7 +537,7 @@ TkTextMakeCharIndex(
 		index += offset;
 	    }
 	} else {
-	    if (charIndex < segPtr->size) {
+	    if (charIndex < (int)segPtr->size) {
 		indexPtr->byteIndex = index;
 		break;
 	    }
@@ -569,11 +571,11 @@ TkTextMakeCharIndex(
 TkTextSegment *
 TkTextIndexToSeg(
     const TkTextIndex *indexPtr,/* Text index. */
-    int *offsetPtr)		/* Where to store offset within segment, or
+    Tcl_Size *offsetPtr)		/* Where to store offset within segment, or
 				 * NULL if offset isn't wanted. */
 {
     TkTextSegment *segPtr;
-    int offset;
+    Tcl_Size offset;
 
     for (offset = indexPtr->byteIndex, segPtr = indexPtr->linePtr->segPtr;
 	    offset >= segPtr->size;
@@ -604,13 +606,13 @@ TkTextIndexToSeg(
  *---------------------------------------------------------------------------
  */
 
-int
+Tcl_Size
 TkTextSegToOffset(
     const TkTextSegment *segPtr,/* Segment whose offset is desired. */
     const TkTextLine *linePtr)	/* Line containing segPtr. */
 {
     const TkTextSegment *segPtr2;
-    int offset = 0;
+    Tcl_Size offset = 0;
 
     for (segPtr2 = linePtr->segPtr; segPtr2 != segPtr;
 	    segPtr2 = segPtr2->nextPtr) {
@@ -790,7 +792,7 @@ GetIndex(
      */
 
     Tcl_DStringInit(&copy);
-    p = strrchr(Tcl_DStringAppend(&copy, string, -1), '.');
+    p = strrchr(Tcl_DStringAppend(&copy, string, TCL_INDEX_NONE), '.');
     if (p != NULL) {
 	TkTextSearch search;
 	TkTextTag *tagPtr;
@@ -842,7 +844,7 @@ GetIndex(
 		    "text doesn't contain any characters tagged with \"%s\"",
 		    tagName));
 	    Tcl_SetErrorCode(interp, "TK", "LOOKUP", "TEXT_INDEX", tagName,
-		    NULL);
+		    (char *)NULL);
 	    Tcl_DStringFree(&copy);
 	    return TCL_ERROR;
 	}
@@ -1007,7 +1009,7 @@ GetIndex(
   error:
     Tcl_DStringFree(&copy);
     Tcl_SetObjResult(interp, Tcl_ObjPrintf("bad text index \"%s\"", string));
-    Tcl_SetErrorCode(interp, "TK", "TEXT", "BAD_INDEX", NULL);
+    Tcl_SetErrorCode(interp, "TK", "TEXT", "BAD_INDEX", (char *)NULL);
     return TCL_ERROR;
 }
 
@@ -1090,7 +1092,7 @@ TkTextIndexAdjustToStartEnd(
  *---------------------------------------------------------------------------
  */
 
-int
+Tcl_Size
 TkTextPrintIndex(
     const TkText *textPtr,
     const TkTextIndex *indexPtr,/* Pointer to index. */
@@ -1099,7 +1101,7 @@ TkTextPrintIndex(
 {
     TkTextSegment *segPtr;
     TkTextLine *linePtr;
-    int numBytes, charIndex;
+    Tcl_Size numBytes, charIndex;
 
     numBytes = indexPtr->byteIndex;
     charIndex = 0;
@@ -1132,7 +1134,7 @@ TkTextPrintIndex(
 	charIndex += numBytes;
     }
 
-    return snprintf(string, TK_POS_CHARS, "%d.%d",
+    return snprintf(string, TK_POS_CHARS, "%d.%" TCL_SIZE_MODIFIER "d",
 	    TkBTreeLinesTo(textPtr, indexPtr->linePtr) + 1, charIndex);
 }
 
@@ -1377,7 +1379,7 @@ ForwBack(
 	    } else {
 		TkTextFindDisplayLineEnd(textPtr, indexPtr, 0, &xOffset);
 		while (count-- > 0) {
-                    TkTextIndex indexPtr2;
+		    TkTextIndex indexPtr2;
 
 		    /*
 		     * Go to the beginning of the line, then backward one
@@ -1388,20 +1390,20 @@ ForwBack(
 		    TkTextIndexBackChars(textPtr, indexPtr, 1, &indexPtr2,
 			    COUNT_DISPLAY_INDICES);
 
-                    /*
-                     * If we couldn't go to the previous line, then we wanted
-                       to go before the start of the text: arrange for returning
-                       the first index of the first display line.
-                     */
+		    /*
+		     * If we couldn't go to the previous line, then we wanted
+		       to go before the start of the text: arrange for returning
+		       the first index of the first display line.
+		     */
 
-                    if (!TkTextIndexCmp(indexPtr, &indexPtr2)) {
-                        xOffset = 0;
-                        break;
-                    }
-                    *indexPtr = indexPtr2;
+		    if (!TkTextIndexCmp(indexPtr, &indexPtr2)) {
+			xOffset = 0;
+			break;
+		    }
+		    *indexPtr = indexPtr2;
 		}
 	    }
-            TkTextFindDisplayLineEnd(textPtr, indexPtr, 0, NULL);
+	    TkTextFindDisplayLineEnd(textPtr, indexPtr, 0, NULL);
 
 	    /*
 	     * This call assumes indexPtr is the beginning of a display line
@@ -1474,7 +1476,7 @@ int
 TkTextIndexForwBytes(
     const TkText *textPtr,
     const TkTextIndex *srcPtr,	/* Source index. */
-    int byteCount,		/* How many bytes forward to move. May be
+    Tcl_Size byteCount,		/* How many bytes forward to move. May be
 				 * negative. */
     TkTextIndex *dstPtr)	/* Destination index: gets modified. */
 {
@@ -1554,7 +1556,7 @@ TkTextIndexForwChars(
     TkTextLine *linePtr;
     TkTextSegment *segPtr;
     TkTextElideInfo *infoPtr = NULL;
-    int byteOffset;
+    Tcl_Size byteOffset;
     char *start, *end, *p;
     int ch;
     int elide = 0;
@@ -1611,7 +1613,7 @@ TkTextIndexForwChars(
 		 * toggled off), or it's a new tag with higher priority.
 		 */
 
-		if (tagPtr->elideString != NULL) {
+		if (tagPtr->elide >= 0) {
 		    infoPtr->tagCnts[tagPtr->priority]++;
 		    if (infoPtr->tagCnts[tagPtr->priority] & 1) {
 			infoPtr->tagPtrs[tagPtr->priority] = tagPtr;
@@ -1639,12 +1641,12 @@ TkTextIndexForwChars(
 				if (infoPtr->tagCnts[infoPtr->elidePriority]
 					& 1) {
 				    elide = infoPtr->tagPtrs
-					    [infoPtr->elidePriority]->elide;
+					    [infoPtr->elidePriority]->elide > 0;
 				    break;
 				}
 			    }
 			} else {
-			    elide = tagPtr->elide;
+			    elide = tagPtr->elide > 0;
 			    infoPtr->elidePriority = tagPtr->priority;
 			}
 		    }
@@ -1655,7 +1657,7 @@ TkTextIndexForwChars(
 		if (segPtr->typePtr == &tkTextCharType) {
 		    start = segPtr->body.chars + byteOffset;
 		    end = segPtr->body.chars + segPtr->size;
-		    for (p = start; p < end; p += TkUtfToUniChar(p, &ch)) {
+		    for (p = start; p < end; p += Tcl_UtfToUniChar(p, &ch)) {
 			if (charCount == 0) {
 			    dstPtr->byteIndex += (p - start);
 			    goto forwardCharDone;
@@ -1742,12 +1744,12 @@ IndexCountBytesOrdered(
 				/* Index describing location of last character
 				 * at which to stop the count. */
 {
-    int byteCount, offset;
+    Tcl_Size byteCount, offset;
     TkTextSegment *segPtr, *segPtr1;
     TkTextLine *linePtr;
 
     if (indexPtr1->linePtr == indexPtr2->linePtr) {
-        return indexPtr2->byteIndex - indexPtr1->byteIndex;
+	return indexPtr2->byteIndex - indexPtr1->byteIndex;
     }
 
     /*
@@ -1761,19 +1763,19 @@ IndexCountBytesOrdered(
     segPtr1 = TkTextIndexToSeg(indexPtr1, &offset);
     byteCount = -offset;
     for (segPtr = segPtr1; segPtr != NULL; segPtr = segPtr->nextPtr) {
-        byteCount += segPtr->size;
+	byteCount += segPtr->size;
     }
 
     linePtr = TkBTreeNextLine(textPtr, indexPtr1->linePtr);
     while (linePtr != indexPtr2->linePtr) {
 	for (segPtr = linePtr->segPtr; segPtr != NULL;
-                segPtr = segPtr->nextPtr) {
-            byteCount += segPtr->size;
-        }
-        linePtr = TkBTreeNextLine(textPtr, linePtr);
-        if (linePtr == NULL) {
-            Tcl_Panic("TextIndexCountBytesOrdered ran out of lines");
-        }
+		segPtr = segPtr->nextPtr) {
+	    byteCount += segPtr->size;
+	}
+	linePtr = TkBTreeNextLine(textPtr, linePtr);
+	if (linePtr == NULL) {
+	    Tcl_Panic("TextIndexCountBytesOrdered ran out of lines");
+	}
     }
 
     byteCount += indexPtr2->byteIndex;
@@ -1819,7 +1821,8 @@ TkTextIndexCount(
     TkTextLine *linePtr1;
     TkTextSegment *segPtr, *seg2Ptr = NULL;
     TkTextElideInfo *infoPtr = NULL;
-    int byteOffset, maxBytes, count = 0, elide = 0;
+    Tcl_Size byteOffset, maxBytes, count = 0;
+    int elide = 0;
     int checkElided = (type & COUNT_DISPLAY);
 
     /*
@@ -1860,7 +1863,7 @@ TkTextIndexCount(
 		     * toggled off), or it's a new tag with higher priority.
 		     */
 
-		    if (tagPtr->elideString != NULL) {
+		    if (tagPtr->elide >= 0) {
 			infoPtr->tagCnts[tagPtr->priority]++;
 			if (infoPtr->tagCnts[tagPtr->priority] & 1) {
 			    infoPtr->tagPtrs[tagPtr->priority] = tagPtr;
@@ -1888,12 +1891,12 @@ TkTextIndexCount(
 				    if (infoPtr->tagCnts[
 					    infoPtr->elidePriority] & 1) {
 					elide = infoPtr->tagPtrs[
-						infoPtr->elidePriority]->elide;
+						infoPtr->elidePriority]->elide > 0;
 					break;
 				    }
 				}
 			    } else {
-				elide = tagPtr->elide;
+				elide = tagPtr->elide > 0;
 				infoPtr->elidePriority = tagPtr->priority;
 			    }
 			}
@@ -1909,10 +1912,10 @@ TkTextIndexCount(
 	    }
 
 	    if (segPtr->typePtr == &tkTextCharType) {
-		int byteLen = segPtr->size - byteOffset;
+		Tcl_Size byteLen = segPtr->size - byteOffset;
 		unsigned char *str = (unsigned char *)
 			segPtr->body.chars + byteOffset;
-		int i;
+		Tcl_Size i;
 
 		if (segPtr == seg2Ptr) {
 		    if (byteLen + byteOffset > maxBytes) {
@@ -1942,7 +1945,7 @@ TkTextIndexCount(
 		}
 	    } else {
 		if (type & COUNT_INDICES) {
-		    int byteLen = segPtr->size - byteOffset;
+		    Tcl_Size byteLen = segPtr->size - byteOffset;
 
 		    if (segPtr == seg2Ptr) {
 			if (byteLen + byteOffset > maxBytes) {
@@ -2004,7 +2007,7 @@ int
 TkTextIndexBackBytes(
     const TkText *textPtr,
     const TkTextIndex *srcPtr,	/* Source index. */
-    int byteCount,		/* How many bytes backward to move. May be
+    Tcl_Size byteCount,		/* How many bytes backward to move. May be
 				 * negative. */
     TkTextIndex *dstPtr)	/* Destination index: gets modified. */
 {
@@ -2121,7 +2124,7 @@ TkTextIndexBackChars(
 		linePtr = TkBTreeNextLine(NULL, linePtr);
 		segPtr = linePtr->segPtr;
 	    }
-	    if (segSize <= segPtr->size) {
+	    if (segSize <= (int)segPtr->size) {
 		break;
 	    }
 	    segSize -= segPtr->size;
@@ -2149,7 +2152,7 @@ TkTextIndexBackChars(
 	     * it's a new tag with higher priority.
 	     */
 
-	    if (tagPtr->elideString != NULL) {
+	    if (tagPtr->elide >= 0) {
 		infoPtr->tagCnts[tagPtr->priority]++;
 		if (infoPtr->tagCnts[tagPtr->priority] & 1) {
 		    infoPtr->tagPtrs[tagPtr->priority] = tagPtr;
@@ -2175,12 +2178,12 @@ TkTextIndexBackChars(
 			while (--infoPtr->elidePriority > 0) {
 			    if (infoPtr->tagCnts[infoPtr->elidePriority] & 1) {
 				elide = infoPtr->tagPtrs[
-					infoPtr->elidePriority]->elide;
+					infoPtr->elidePriority]->elide > 0;
 				break;
 			    }
 			}
 		    } else {
-			elide = tagPtr->elide;
+			elide = tagPtr->elide > 0;
 			infoPtr->elidePriority = tagPtr->priority;
 		    }
 		}
@@ -2191,7 +2194,7 @@ TkTextIndexBackChars(
 	    if (segPtr->typePtr == &tkTextCharType) {
 		start = segPtr->body.chars;
 		end = segPtr->body.chars + segSize;
-		for (p = end; ; p = TkUtfPrev(p, start)) {
+		for (p = end; ; p = Tcl_UtfPrev(p, start)) {
 		    if (charCount == 0) {
 			dstPtr->byteIndex -= (end - p);
 			goto backwardCharDone;
@@ -2199,7 +2202,12 @@ TkTextIndexBackChars(
 		    if (p == start) {
 			break;
 		    }
-		    charCount--;
+		    if ((sizeof(Tcl_UniChar) == 2) &&  (unsigned)(UCHAR(*p) - 0xF0) <= 5) {
+			charCount--; /* Characters > U+FFFF count as 2 here */
+		    }
+		    if (charCount != 0) {
+			charCount--;
+		    }
 		}
 	    } else {
 		if (type & COUNT_INDICES) {
@@ -2362,7 +2370,7 @@ StartEnd(
     } else if ((*string == 'w') && (strncmp(string, "wordend", length) == 0)
 	    && (length >= 5)) {
 	int firstChar = 1;
-	int offset;
+	Tcl_Size offset;
 
 	/*
 	 * If the current character isn't part of a word then just move
@@ -2381,7 +2389,7 @@ StartEnd(
 	    if (segPtr->typePtr == &tkTextCharType) {
 		int ch;
 
-		chSize = TkUtfToUniChar(segPtr->body.chars + offset, &ch);
+		chSize = Tcl_UtfToUniChar(segPtr->body.chars + offset, &ch);
 		if (!Tcl_UniCharIsWordChar(ch)) {
 		    break;
 		}
@@ -2405,7 +2413,7 @@ StartEnd(
     } else if ((*string == 'w') && (strncmp(string, "wordstart", length) == 0)
 	    && (length >= 5)) {
 	int firstChar = 1;
-	int offset;
+	Tcl_Size offset;
 
 	if (modifier == TKINDEX_DISPLAY) {
 	    TkTextIndexForwChars(textPtr, indexPtr, 0, indexPtr,
@@ -2426,32 +2434,32 @@ StartEnd(
 	    if (segPtr->typePtr == &tkTextCharType) {
 
 		int ch;
-		TkUtfToUniChar(segPtr->body.chars + offset, &ch);
+		Tcl_UtfToUniChar(segPtr->body.chars + offset, &ch);
 		if (!Tcl_UniCharIsWordChar(ch)) {
 		    break;
 		}
-		if (offset > 0) {
+		if (offset + 1 > 1) {
 		    chSize = (segPtr->body.chars + offset
-			    - TkUtfPrev(segPtr->body.chars + offset,
+			    - Tcl_UtfPrev(segPtr->body.chars + offset,
 			    segPtr->body.chars));
 		}
 		firstChar = 0;
 	    }
-            if (offset == 0) {
+	    if (offset == 0) {
 		if (indexPtr->byteIndex == 0) {
 		    goto done;
 		}
-                if (modifier == TKINDEX_DISPLAY) {
-                    TkTextIndexBackChars(textPtr, indexPtr, 1, indexPtr,
-                        COUNT_DISPLAY_INDICES);
-                } else {
-                    TkTextIndexBackChars(NULL, indexPtr, 1, indexPtr,
-                        COUNT_INDICES);
-                }
-            } else {
-                indexPtr->byteIndex -= chSize;
-            }
-            offset -= chSize;
+		if (modifier == TKINDEX_DISPLAY) {
+		    TkTextIndexBackChars(textPtr, indexPtr, 1, indexPtr,
+			    COUNT_DISPLAY_INDICES);
+		} else {
+		    TkTextIndexBackChars(NULL, indexPtr, 1, indexPtr,
+			    COUNT_INDICES);
+		}
+	    } else {
+		indexPtr->byteIndex -= chSize;
+	    }
+	    offset -= chSize;
 	    if (offset < 0) {
 		segPtr = TkTextIndexToSeg(indexPtr, &offset);
 	    }
