@@ -7,22 +7,22 @@
  *
  * Scrollable interface:
  *
- * 	+ 'first' is controlled by [xy]view widget command
- * 	  and other scrolling commands like 'see';
+ *	+ 'first' is controlled by [xy]view widget command
+ *	  and other scrolling commands like 'see';
  *      + 'total' depends on widget contents;
  *      + 'last' depends on first, total, and widget size.
  *
  * Choreography (typical usage):
  *
- * 	1. User adjusts scrollbar, scrollbar widget calls its -command
- * 	2. Scrollbar -command invokes the scrollee [xy]view widget method
- * 	3. TtkScrollviewCommand calls TtkScrollTo(), which updates
- * 	   'first' and schedules a redisplay.
- * 	4. Once the scrollee knows 'total' and 'last' (typically in
- * 	   the LayoutProc), call TtkScrolled(h,first,last,total) to
- * 	   synchronize the scrollbar.
- * 	5. The scrollee -[xy]scrollcommand is called (in an idle callback)
- * 	6. Which calls the scrollbar 'set' method and redisplays the scrollbar.
+ *	1. User adjusts scrollbar, scrollbar widget calls its -command
+ *	2. Scrollbar -command invokes the scrollee [xy]view widget method
+ *	3. TtkScrollviewCommand calls TtkScrollTo(), which updates
+ *	   'first' and schedules a redisplay.
+ *	4. Once the scrollee knows 'total' and 'last' (typically in
+ *	   the LayoutProc), call TtkScrolled(h,first,last,total) to
+ *	   synchronize the scrollbar.
+ *	5. The scrollee -[xy]scrollcommand is called (in an idle callback)
+ *	6. Which calls the scrollbar 'set' method and redisplays the scrollbar.
  *
  * If the scrollee has internal scrolling (e.g., a 'see' method),
  * it should TtkScrollTo() directly (step 2).
@@ -45,13 +45,13 @@
 
 struct ScrollHandleRec
 {
-    unsigned 	flags;
+    unsigned	flags;
     WidgetCore	*corePtr;
     Scrollable	*scrollPtr;
 };
 
 /* TtkCreateScrollHandle --
- * 	Initialize scroll handle.
+ *	Initialize scroll handle.
  */
 ScrollHandle TtkCreateScrollHandle(WidgetCore *corePtr, Scrollable *scrollPtr)
 {
@@ -69,7 +69,7 @@ ScrollHandle TtkCreateScrollHandle(WidgetCore *corePtr, Scrollable *scrollPtr)
 
 /* UpdateScrollbar --
  *	Call the -scrollcommand callback to sync the scrollbar.
- * 	Returns: Whatever the -scrollcommand does.
+ *	Returns: Whatever the -scrollcommand does.
  */
 static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
 {
@@ -82,7 +82,7 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
 
     h->flags &= ~SCROLL_UPDATE_REQUIRED;
 
-    if (s->scrollCmd == NULL) {
+    if (s->scrollCmdObj == NULL) {
 	return TCL_OK;
     }
 
@@ -90,7 +90,7 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
     Tcl_PrintDouble(interp, (double)s->first / s->total, arg1+1);
     Tcl_PrintDouble(interp, (double)s->last / s->total, arg2+1);
     Tcl_DStringInit(&buf);
-    Tcl_DStringAppend(&buf, s->scrollCmd, TCL_INDEX_NONE);
+    Tcl_DStringAppend(&buf, Tcl_GetString(s->scrollCmdObj), TCL_INDEX_NONE);
     Tcl_DStringAppend(&buf, arg1, TCL_INDEX_NONE);
     Tcl_DStringAppend(&buf, arg2, TCL_INDEX_NONE);
 
@@ -121,7 +121,7 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
 }
 
 /* UpdateScrollbarBG --
- * 	Idle handler to update the scrollbar.
+ *	Idle handler to update the scrollbar.
  */
 static void UpdateScrollbarBG(void *clientData)
 {
@@ -139,7 +139,7 @@ static void UpdateScrollbarBG(void *clientData)
 }
 
 /* TtkScrolled --
- * 	Update scroll info, schedule scrollbar update.
+ *	Update scroll info, schedule scrollbar update.
  */
 void TtkScrolled(ScrollHandle h, int first, int last, int total)
 {
@@ -174,9 +174,9 @@ void TtkScrolled(ScrollHandle h, int first, int last, int total)
 }
 
 /* TtkScrollbarUpdateRequired --
- * 	Force a scrollbar update at the next call to TtkScrolled(),
- * 	even if scroll parameters haven't changed (e.g., if
- * 	-yscrollcommand has changed).
+ *	Force a scrollbar update at the next call to TtkScrolled(),
+ *	even if scroll parameters haven't changed (e.g., if
+ *	-yscrollcommand has changed).
  */
 
 void TtkScrollbarUpdateRequired(ScrollHandle h)
@@ -185,9 +185,9 @@ void TtkScrollbarUpdateRequired(ScrollHandle h)
 }
 
 /* TtkUpdateScrollInfo --
- * 	Call the layoutProc to update the scroll info first, last, and total.
- * 	Do it only if needed, that is when a redisplay is pending (which
- * 	indicates scroll info are possibly out of date).
+ *	Call the layoutProc to update the scroll info first, last, and total.
+ *	Do it only if needed, that is when a redisplay is pending (which
+ *	indicates scroll info are possibly out of date).
  */
 
 void TtkUpdateScrollInfo(ScrollHandle h)
@@ -198,7 +198,7 @@ void TtkUpdateScrollInfo(ScrollHandle h)
 }
 
 /* TtkScrollviewCommand --
- * 	Widget [xy]view command implementation.
+ *	Widget [xy]view command implementation.
  *
  *  $w [xy]view -- return current view region
  *  $w [xy]view $index -- set topmost item
