@@ -1,7 +1,7 @@
 /*
  * Theme engine: private definitions.
  *
- * Copyright (c) 2004 Joe English.  Freely redistributable.
+ * Copyright © 2004 Joe English.  Freely redistributable.
  */
 
 #ifndef _TTKTHEMEINT
@@ -9,17 +9,26 @@
 
 #include "ttkTheme.h"
 
+/*------------------------------------------------------------------------
+ * +++ Widget states (internal).
+ */
+
+#define TTK_STATE_OPEN		(1<<16)
+#define TTK_STATE_LEAF		(1<<17)
+#define TTK_STATE_FIRST		(1<<18)
+#define TTK_STATE_LAST		(1<<19)
+
 typedef struct Ttk_TemplateNode_ Ttk_TemplateNode, *Ttk_LayoutTemplate;
 
 MODULE_SCOPE Ttk_ElementClass *Ttk_GetElement(Ttk_Theme, const char *name);
 MODULE_SCOPE const char *Ttk_ElementClassName(Ttk_ElementClass *);
 
 MODULE_SCOPE void Ttk_ElementSize(
-	Ttk_ElementClass *, Ttk_Style, char *recordPtr, Tk_OptionTable,
+	Ttk_ElementClass *, Ttk_Style, void *recordPtr, Tk_OptionTable,
 	Tk_Window tkwin, Ttk_State state,
 	int *widthPtr, int *heightPtr, Ttk_Padding*);
 MODULE_SCOPE void Ttk_DrawElement(
-	Ttk_ElementClass *, Ttk_Style, char *recordPtr, Tk_OptionTable,
+	Ttk_ElementClass *, Ttk_Style, void *recordPtr, Tk_OptionTable,
 	Tk_Window tkwin, Drawable d, Ttk_Box b, Ttk_State state);
 
 MODULE_SCOPE Tcl_Obj *Ttk_QueryStyle(
@@ -39,4 +48,10 @@ MODULE_SCOPE Ttk_LayoutTemplate Ttk_FindLayoutTemplate(
 
 MODULE_SCOPE const char *Ttk_StyleName(Ttk_Style);
 
+MODULE_SCOPE void TtkSetBlinkCursorTimes(Tcl_Interp* interp);
+
+MODULE_SCOPE int TtkBoxEqual(Ttk_Box, Ttk_Box);
+
+#define TTK_OPTION_UNDERLINE_DEF(type, field) NULL, offsetof(type, field), TCL_INDEX_NONE, TK_OPTION_NULL_OK, NULL
+
 #endif /* _TTKTHEMEINT */
