@@ -4,9 +4,9 @@
 # checkbutton, and radiobutton widgets and provides procedures
 # that help in implementing those bindings.
 #
-# Copyright (c) 1992-1994 The Regents of the University of California.
-# Copyright (c) 1994-1996 Sun Microsystems, Inc.
-# Copyright (c) 2002 ActiveState Corporation.
+# Copyright © 1992-1994 The Regents of the University of California.
+# Copyright © 1994-1996 Sun Microsystems, Inc.
+# Copyright © 2002 ActiveState Corporation.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -17,10 +17,11 @@
 #-------------------------------------------------------------------------
 
 if {[tk windowingsystem] eq "aqua"} {
+
     bind Radiobutton <Enter> {
 	tk::ButtonEnter %W
     }
-    bind Radiobutton <1> {
+    bind Radiobutton <Button-1> {
 	tk::ButtonDown %W
     }
     bind Radiobutton <ButtonRelease-1> {
@@ -29,24 +30,27 @@ if {[tk windowingsystem] eq "aqua"} {
     bind Checkbutton <Enter> {
 	tk::ButtonEnter %W
     }
-    bind Checkbutton <1> {
+    bind Checkbutton <Button-1> {
 	tk::ButtonDown %W
     }
     bind Checkbutton <ButtonRelease-1> {
 	tk::ButtonUp %W
     }
+    bind Checkbutton <Leave> {
+	tk::ButtonLeave %W
+    }
 }
 if {"win32" eq [tk windowingsystem]} {
-    bind Checkbutton <equal> {
+    bind Checkbutton <=> {
 	tk::CheckRadioInvoke %W select
     }
-    bind Checkbutton <plus> {
+    bind Checkbutton <+> {
 	tk::CheckRadioInvoke %W select
     }
     bind Checkbutton <minus> {
 	tk::CheckRadioInvoke %W deselect
     }
-    bind Checkbutton <1> {
+    bind Checkbutton <Button-1> {
 	tk::CheckRadioDown %W
     }
     bind Checkbutton <ButtonRelease-1> {
@@ -55,8 +59,11 @@ if {"win32" eq [tk windowingsystem]} {
     bind Checkbutton <Enter> {
 	tk::CheckRadioEnter %W
     }
+    bind Checkbutton <Leave> {
+	tk::ButtonLeave %W
+    }
 
-    bind Radiobutton <1> {
+    bind Radiobutton <Button-1> {
 	tk::CheckRadioDown %W
     }
     bind Radiobutton <ButtonRelease-1> {
@@ -69,7 +76,7 @@ if {"win32" eq [tk windowingsystem]} {
 if {"x11" eq [tk windowingsystem]} {
     bind Checkbutton <Return> {
 	if {!$tk_strictMotif} {
-	    tk::CheckRadioInvoke %W
+	    tk::CheckInvoke %W
 	}
     }
     bind Radiobutton <Return> {
@@ -77,18 +84,21 @@ if {"x11" eq [tk windowingsystem]} {
 	    tk::CheckRadioInvoke %W
 	}
     }
-    bind Checkbutton <1> {
-	tk::CheckRadioInvoke %W
+    bind Checkbutton <Button-1> {
+	tk::CheckInvoke %W
     }
-    bind Radiobutton <1> {
+    bind Radiobutton <Button-1> {
 	tk::CheckRadioInvoke %W
     }
     bind Checkbutton <Enter> {
-	tk::ButtonEnter %W
+	tk::CheckEnter %W
     }
     bind Radiobutton <Enter> {
 	tk::ButtonEnter %W
     }
+    bind Checkbutton <Leave> {
+	tk::CheckLeave %W
+    }
 }
 
 bind Button <space> {
@@ -100,6 +110,15 @@ bind Checkbutton <space> {
 bind Radiobutton <space> {
     tk::CheckRadioInvoke %W
 }
+bind Button <<Invoke>> {
+    tk::ButtonInvoke %W
+}
+bind Checkbutton <<Invoke>> {
+    tk::CheckRadioInvoke %W
+}
+bind Radiobutton <<Invoke>> {
+    tk::CheckRadioInvoke %W
+}
 
 bind Button <FocusIn> {}
 bind Button <Enter> {
@@ -108,7 +127,7 @@ bind Button <Enter> {
 bind Button <Leave> {
     tk::ButtonLeave %W
 }
-bind Button <1> {
+bind Button <Button-1> {
     tk::ButtonDown %W
 }
 bind Button <ButtonRelease-1> {
@@ -116,9 +135,6 @@ bind Button <ButtonRelease-1> {
 }
 
 bind Checkbutton <FocusIn> {}
-bind Checkbutton <Leave> {
-    tk::ButtonLeave %W
-}
 
 bind Radiobutton <FocusIn> {}
 bind Radiobutton <Leave> {
@@ -128,7 +144,7 @@ bind Radiobutton <Leave> {
 if {"win32" eq [tk windowingsystem]} {
 
 #########################
-# Windows implementation 
+# Windows implementation
 #########################
 
 # ::tk::ButtonEnter --
@@ -581,12 +597,25 @@ proc ::tk::ButtonUp w {
 # w -		The name of the widget.
 
 proc ::tk::ButtonInvoke w {
-    if {[$w cget -state] ne "disabled"} {
+    if {[winfo exists $w] && [$w cget -state] ne "disabled"} {
 	set oldRelief [$w cget -relief]
 	set oldState [$w cget -state]
 	$w configure -state active -relief sunken
-	update idletasks
-	after 100
+	after 100 [list ::tk::ButtonInvokeEnd $w $oldState $oldRelief]
+    }
+}
+
+# ::tk::ButtonInvokeEnd --
+# The procedure below is called after a button is invoked through
+# the keyboard.  It simulate a release of the button via the mouse.
+#
+# Arguments:
+# w -         The name of the widget.
+# oldState -  Old state to be set back.
+# oldRelief - Old relief to be set back.
+
+proc ::tk::ButtonInvokeEnd {w oldState oldRelief} {
+    if {[winfo exists $w]} {
 	$w configure -state $oldState -relief $oldRelief
 	uplevel #0 [list $w invoke]
     }
@@ -633,3 +662,121 @@ proc ::tk::CheckRadioInvoke {w {cmd invoke}} {
 	uplevel #0 [list $w $cmd]
     }
 }
+
+# Special versions of the handlers for checkbuttons on Unix that do the magic
+# to make things work right when the checkbutton indicator is hidden;
+# radiobuttons don't need this complexity.
+
+# ::tk::CheckInvoke --
+# The procedure below invokes the checkbutton, like ButtonInvoke, but handles
+# what to do when the checkbutton indicator is missing. Only used on Unix.
+#
+# Arguments:
+# w -		The name of the widget.
+
+proc ::tk::CheckInvoke {w} {
+    variable ::tk::Priv
+    if {[$w cget -state] ne "disabled"} {
+	# Additional logic to switch the "selected" colors around if necessary
+	# (when we're indicator-less).
+
+	if {![$w cget -indicatoron] && [info exist Priv($w,selectcolor)]} {
+	    if {[$w cget -selectcolor] eq $Priv($w,aselectcolor)} {
+		$w configure -selectcolor $Priv($w,selectcolor)
+	    } else {
+		$w configure -selectcolor $Priv($w,aselectcolor)
+	    }
+	}
+	uplevel #0 [list $w invoke]
+    }
+}
+
+# ::tk::CheckEnter --
+# The procedure below enters the checkbutton, like ButtonEnter, but handles
+# what to do when the checkbutton indicator is missing. Only used on Unix.
+#
+# Arguments:
+# w -		The name of the widget.
+
+proc ::tk::CheckEnter {w} {
+    variable ::tk::Priv
+    if {[$w cget -state] ne "disabled"} {
+	# On unix the state is active just with mouse-over
+	$w configure -state active
+
+	# If the mouse button is down, set the relief to sunken on entry.
+	# Overwise, if there's an -overrelief value, set the relief to that.
+
+	set Priv($w,relief) [$w cget -relief]
+	if {$Priv(buttonWindow) eq $w} {
+	    $w configure -relief sunken
+	    set Priv($w,prelief) sunken
+	} elseif {[set over [$w cget -overrelief]] ne ""} {
+	    $w configure -relief $over
+	    set Priv($w,prelief) $over
+	}
+
+	# Compute what the "selected and active" color should be.
+
+	if {![$w cget -indicatoron] && [$w cget -selectcolor] ne ""} {
+	    set Priv($w,selectcolor) [$w cget -selectcolor]
+	    lassign [winfo rgb $w [$w cget -selectcolor]]      r1 g1 b1
+	    lassign [winfo rgb $w [$w cget -activebackground]] r2 g2 b2
+	    set Priv($w,aselectcolor) \
+		[format "#%04x%04x%04x" [expr {($r1+$r2)/2}] \
+		     [expr {($g1+$g2)/2}] [expr {($b1+$b2)/2}]]
+	    # use uplevel to work with other var resolvers
+	    if {[uplevel #0 [list set [$w cget -variable]]]
+		 eq [$w cget -onvalue]} {
+		$w configure -selectcolor $Priv($w,aselectcolor)
+	    }
+	}
+    }
+    set Priv(window) $w
+}
+
+# ::tk::CheckLeave --
+# The procedure below leaves the checkbutton, like ButtonLeave, but handles
+# what to do when the checkbutton indicator is missing. Only used on Unix.
+#
+# Arguments:
+# w -		The name of the widget.
+
+proc ::tk::CheckLeave {w} {
+    variable ::tk::Priv
+    if {[$w cget -state] ne "disabled"} {
+	$w configure -state normal
+    }
+
+    # Restore the original button "selected" color; but only if the user
+    # has not changed it in the meantime.
+
+    if {![$w cget -indicatoron] && [info exist Priv($w,selectcolor)]} {
+	if {[$w cget -selectcolor] eq $Priv($w,selectcolor)
+		|| ([info exist Priv($w,aselectcolor)] &&
+		    [$w cget -selectcolor] eq $Priv($w,aselectcolor))} {
+	    $w configure -selectcolor $Priv($w,selectcolor)
+	}
+    }
+    unset -nocomplain Priv($w,selectcolor) Priv($w,aselectcolor)
+
+    # Restore the original button relief if it was changed by Tk. That is
+    # signaled by the existence of Priv($w,prelief).
+
+    if {[info exists Priv($w,relief)]} {
+	if {[info exists Priv($w,prelief)] && \
+		$Priv($w,prelief) eq [$w cget -relief]} {
+	    $w configure -relief $Priv($w,relief)
+	}
+	unset -nocomplain Priv($w,relief) Priv($w,prelief)
+    }
+
+    set Priv(window) ""
+}
+
+return
+
+# Local Variables:
+# mode: tcl
+# fill-column: 78
+# End:
