@@ -6,7 +6,7 @@ if {![info exists widgetDemo]} {
     error "This script should be run from the \"widget\" demo."
 }
 
-package require Tk
+package require tk
 
 set w .spin
 catch {destroy $w}
@@ -38,16 +38,8 @@ set australianCities {
 }
 
 spinbox $w.s1 -from 1 -to 10 -width 10 -validate key \
-	-vcmd {string is integer %P}
+	-validatecommand {string is integer %P}
 spinbox $w.s2 -from 0 -to 3 -increment .5 -format %05.2f -width 10
 spinbox $w.s3 -values $australianCities -width 10
 
-#entry $w.e1
-#entry $w.e2
-#entry $w.e3
-pack $w.s1 $w.s2 $w.s3 -side top -pady 5 -padx 10 ;#-fill x
-
-#$w.e1 insert 0 "Initial value"
-#$w.e2 insert end "This entry contains a long value, much too long "
-#$w.e2 insert end "to fit in the window at one time, so long in fact "
-#$w.e2 insert end "that you'll have to scan or scroll to see the end."
+pack $w.s1 $w.s2 $w.s3 -side top -pady 3p -padx 7.5p
