@@ -3,7 +3,7 @@
 # This file defines several procedures for managing the input
 # focus.
 #
-# Copyright (c) 1994-1995 Sun Microsystems, Inc.
+# Copyright © 1994-1995 Sun Microsystems, Inc.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -36,7 +36,7 @@ proc ::tk_focusNext w {
 	    incr i
 	    if {$i < [llength $children]} {
 		set cur [lindex $children $i]
-              if {[winfo toplevel $cur] eq $cur} {
+		if {[winfo toplevel $cur] eq $cur} {
 		    continue
 		} else {
 		    break
@@ -163,8 +163,7 @@ proc ::tk::FocusOK w {
 proc ::tk_focusFollowsMouse {} {
     set old [bind all <Enter>]
     set script {
-	if {"%d" eq "NotifyAncestor" \
-		|| "%d" eq "NotifyNonlinear" \
+	if {"%d" eq "NotifyAncestor" || "%d" eq "NotifyNonlinear" \
 		|| "%d" eq "NotifyInferior"} {
 	    if {[tk::FocusOK %W]} {
 		focus %W
