@@ -4,10 +4,10 @@
  *	This file defines the defaults for all options for all of
  *	the Tk widgets.
  *
- * Copyright (c) 1991-1994 The Regents of the University of California.
- * Copyright (c) 1994-1997 Sun Microsystems, Inc.
- * Copyright 2001-2009, Apple Inc.
- * Copyright (c) 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright © 1991-1994 The Regents of the University of California.
+ * Copyright © 1994-1997 Sun Microsystems, Inc.
+ * Copyright © 2001-2009 Apple Inc.
+ * Copyright © 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -45,7 +45,7 @@
 #define INACTIVE_SELECT_BG	"systemUnemphasizedSelectedTextBackgroundColor"
 #define TROUGH			"#c3c3c3"
 #define INDICATOR		"#b03060"
-#define DISABLED		"#a3a3a3"
+#define DISABLED		"systemDisabledControlTextColor"
 #define IGNORED                 "#abcdef"
 
 /*
@@ -106,7 +106,6 @@
 #define DEF_BUTTON_TAKE_FOCUS		NULL
 #define DEF_BUTTON_TEXT			""
 #define DEF_BUTTON_TEXT_VARIABLE	""
-#define DEF_BUTTON_UNDERLINE		"-1"
 #define DEF_BUTTON_VALUE		""
 #define DEF_BUTTON_WIDTH		"0"
 #define DEF_BUTTON_WRAP_LENGTH		"0"
@@ -184,6 +183,8 @@
 #define DEF_ENTRY_INSERT_ON_TIME	"600"
 #define DEF_ENTRY_INSERT_WIDTH		"1"
 #define DEF_ENTRY_JUSTIFY		"left"
+#define DEF_ENTRY_PLACEHOLDER		""
+#define DEF_ENTRY_PLACEHOLDERFG		"systemPlaceholderTextColor"
 #define DEF_ENTRY_READONLY_BG_COLOR	NORMAL_BG
 #define DEF_ENTRY_READONLY_BG_MONO	WHITE
 #define DEF_ENTRY_RELIEF		"sunken"
@@ -205,7 +206,9 @@
  */
 
 #define DEF_FRAME_BG_COLOR		NORMAL_BG
+#define DEF_FRAME_BG_IMAGE		NULL
 #define DEF_FRAME_BG_MONO		WHITE
+#define DEF_FRAME_BG_TILE		"0"
 #define DEF_FRAME_BORDER_WIDTH		"0"
 #define DEF_FRAME_CLASS			"Frame"
 #define DEF_FRAME_COLORMAP		""
@@ -293,7 +296,6 @@
 #define DEF_MENU_ENTRY_CHECK_VARIABLE	NULL
 #define DEF_MENU_ENTRY_RADIO_VARIABLE	"selectedButton"
 #define DEF_MENU_ENTRY_SELECT		NULL
-#define DEF_MENU_ENTRY_UNDERLINE	"-1"
 
 /*
  * Defaults for menus overall:
@@ -304,6 +306,7 @@
 #define DEF_MENU_ACTIVE_BORDER_WIDTH	"0"
 #define DEF_MENU_ACTIVE_FG_COLOR	IGNORED
 #define DEF_MENU_ACTIVE_FG_MONO		IGNORED
+#define DEF_MENU_ACTIVE_RELIEF		"flat"
 #define DEF_MENU_BG_COLOR		"#000001" /* Detects custom bg. */
 #define DEF_MENU_BG_MONO		IGNORED
 #define DEF_MENU_BORDER_WIDTH		"0"
@@ -327,12 +330,12 @@
  */
 
 #define DEF_MENUBUTTON_ANCHOR		"w"
-#define DEF_MENUBUTTON_ACTIVE_BG_COLOR	ACTIVE_BG
-#define DEF_MENUBUTTON_ACTIVE_BG_MONO	WHITE
-#define DEF_MENUBUTTON_ACTIVE_FG_COLOR	ACTIVE_FG
-#define DEF_MENUBUTTON_ACTIVE_FG_MONO	BLACK
-#define DEF_MENUBUTTON_BG_COLOR		NORMAL_BG
-#define DEF_MENUBUTTON_BG_MONO		WHITE
+#define DEF_MENUBUTTON_ACTIVE_BG_COLOR	NORMAL_BG /*ignored*/
+#define DEF_MENUBUTTON_ACTIVE_BG_MONO	NORMAL_BG /*ignored*/
+#define DEF_MENUBUTTON_ACTIVE_FG_COLOR	"systemTextColor"
+#define DEF_MENUBUTTON_ACTIVE_FG_MONO	"systemTextColor"
+#define DEF_MENUBUTTON_BG_COLOR		NORMAL_BG /*ignored*/
+#define DEF_MENUBUTTON_BG_MONO		NORMAL_BG /*ignored*/
 #define DEF_MENUBUTTON_BITMAP		""
 #define DEF_MENUBUTTON_BORDER_WIDTH	"0"
 #define DEF_MENUBUTTON_CURSOR		""
@@ -357,7 +360,6 @@
 #define DEF_MENUBUTTON_TAKE_FOCUS	"0"
 #define DEF_MENUBUTTON_TEXT		""
 #define DEF_MENUBUTTON_TEXT_VARIABLE	""
-#define DEF_MENUBUTTON_UNDERLINE	"-1"
 #define DEF_MENUBUTTON_WIDTH		"0"
 #define DEF_MENUBUTTON_WRAP_LENGTH	"0"
 
@@ -377,8 +379,8 @@
 #define DEF_MESSAGE_HIGHLIGHT		NORMAL_FG
 #define DEF_MESSAGE_HIGHLIGHT_WIDTH	"0"
 #define DEF_MESSAGE_JUSTIFY		"left"
-#define DEF_MESSAGE_PADX		"-1"
-#define DEF_MESSAGE_PADY		"-1"
+#define DEF_MESSAGE_PADX		NULL
+#define DEF_MESSAGE_PADY		NULL
 #define DEF_MESSAGE_RELIEF		"flat"
 #define DEF_MESSAGE_TAKE_FOCUS		"0"
 #define DEF_MESSAGE_TEXT		""
@@ -429,7 +431,7 @@
 #define DEF_SCALE_ACTIVE_BG_MONO	BLACK
 #define DEF_SCALE_BG_COLOR		NORMAL_BG
 #define DEF_SCALE_BG_MONO		WHITE
-#define DEF_SCALE_BIG_INCREMENT		"0"
+#define DEF_SCALE_BIG_INCREMENT		"0.0"
 #define DEF_SCALE_BORDER_WIDTH		"1"
 #define DEF_SCALE_COMMAND		""
 #define DEF_SCALE_CURSOR		""
@@ -437,7 +439,7 @@
 #define DEF_SCALE_FONT			"TkDefaultFont"
 #define DEF_SCALE_FG_COLOR		NORMAL_FG
 #define DEF_SCALE_FG_MONO		BLACK
-#define DEF_SCALE_FROM			"0"
+#define DEF_SCALE_FROM			"0.0"
 #define DEF_SCALE_HIGHLIGHT_BG_COLOR	DEF_SCALE_BG_COLOR
 #define DEF_SCALE_HIGHLIGHT_BG_MONO	DEF_SCALE_BG_MONO
 #define DEF_SCALE_HIGHLIGHT		NORMAL_FG
@@ -448,7 +450,7 @@
 #define DEF_SCALE_RELIEF		"flat"
 #define DEF_SCALE_REPEAT_DELAY	"300"
 #define DEF_SCALE_REPEAT_INTERVAL	"100"
-#define DEF_SCALE_RESOLUTION		"1"
+#define DEF_SCALE_RESOLUTION		"1.0"
 #define DEF_SCALE_TROUGH_COLOR		TROUGH
 #define DEF_SCALE_TROUGH_MONO		WHITE
 #define DEF_SCALE_SHOW_VALUE		"1"
@@ -456,8 +458,8 @@
 #define DEF_SCALE_SLIDER_RELIEF		"raised"
 #define DEF_SCALE_STATE			"normal"
 #define DEF_SCALE_TAKE_FOCUS		NULL
-#define DEF_SCALE_TICK_INTERVAL		"0"
-#define DEF_SCALE_TO			"100"
+#define DEF_SCALE_TICK_INTERVAL		"0.0"
+#define DEF_SCALE_TO			"100.0"
 #define DEF_SCALE_VARIABLE		""
 #define DEF_SCALE_WIDTH			"15"
 
@@ -473,7 +475,7 @@
 #define DEF_SCROLLBAR_BORDER_WIDTH	"0"
 #define DEF_SCROLLBAR_COMMAND		""
 #define DEF_SCROLLBAR_CURSOR		""
-#define DEF_SCROLLBAR_EL_BORDER_WIDTH	"-1"
+#define DEF_SCROLLBAR_EL_BORDER_WIDTH	NULL
 #define DEF_SCROLLBAR_HIGHLIGHT_BG	NORMAL_BG
 #define DEF_SCROLLBAR_HIGHLIGHT		NORMAL_FG
 #define DEF_SCROLLBAR_HIGHLIGHT_WIDTH	"0"
@@ -515,7 +517,7 @@
 #define DEF_TEXT_PADX			"1"
 #define DEF_TEXT_PADY			"1"
 #define DEF_TEXT_RELIEF			"flat"
-#define DEF_TEXT_INACTIVE_SELECT_COLOR	INACTIVE_SELECT_BG
+#define DEF_TEXT_INACTIVE_SELECT_BG_COLOR	INACTIVE_SELECT_BG
 #define DEF_TEXT_SELECT_COLOR		SELECT_BG
 #define DEF_TEXT_SELECT_MONO		BLACK
 #define DEF_TEXT_SELECT_BD_COLOR	"1"
