@@ -3,9 +3,9 @@
  *
  *	Declarations of Macintosh specific shared variables and procedures.
  *
- * Copyright (c) 1995-1997 Sun Microsystems, Inc.
- * Copyright 2001-2009, Apple Inc.
- * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright © 1995-1997 Sun Microsystems, Inc.
+ * Copyright © 2001-2009 Apple Inc.
+ * Copyright © 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -24,7 +24,9 @@
 
 #ifndef _TKMAC
 #include "tkMacOSX.h"
+#define Cursor QDCursor
 #import <Cocoa/Cocoa.h>
+#undef Cursor
 #endif
 
 /*
@@ -86,7 +88,6 @@ typedef struct TkWindowPrivate MacDrawable;
 #define TK_DRAWN_UNDER_MENU	0x08
 #define TK_IS_PIXMAP		0x10
 #define TK_IS_BW_PIXMAP		0x20
-#define TK_DO_NOT_DRAW          0x40
 #define TTK_HAS_CONTRASTING_BG  0x80
 
 /*
@@ -98,24 +99,6 @@ typedef struct TkWindowPrivate MacDrawable;
  * Destroy of a child.
  */
 
-/*
- * This structure is for handling Netscape-type in process
- * embedding where Tk does not control the top-level. It contains
- * various functions that are needed by Mac specific routines, like
- * TkMacOSXGetDrawablePort. The definitions of the function types
- * are in tkMacOSX.h.
- */
-
-typedef struct {
-    Tk_MacOSXEmbedRegisterWinProc *registerWinProc;
-    Tk_MacOSXEmbedGetGrafPortProc *getPortProc;
-    Tk_MacOSXEmbedMakeContainerExistProc *containerExistProc;
-    Tk_MacOSXEmbedGetClipProc *getClipProc;
-    Tk_MacOSXEmbedGetOffsetInParentProc *getOffsetProc;
-} TkMacOSXEmbedHandler;
-
-MODULE_SCOPE TkMacOSXEmbedHandler *tkMacOSXEmbedHandler;
-
 /*
  * Undef compatibility platform types defined above.
  */
@@ -176,8 +159,6 @@ MODULE_SCOPE TkMacOSXEmbedHandler *tkMacOSXEmbedHandler;
  */
 
 MODULE_SCOPE void TkMacOSXDefaultStartupScript(void);
-MODULE_SCOPE void TkpClipDrawableToRect(Display *display, Drawable d, int x,
-	int y, int width, int height);
 MODULE_SCOPE Bool TkTestLogDisplay(Drawable drawable);
 
 /*
