@@ -58,8 +58,8 @@ XDestroyWindow(
     MacDrawable *macWin = (MacDrawable *)window;
     TKContentView *view = (TKContentView *)TkMacOSXGetNSViewForDrawable(macWin);
     //fprintf(stderr, "XDestroyWindow: %s with parent %s\n",
-    // 	    Tk_PathName(macWin->winPtr),
-    // 	    Tk_PathName(macWin->winPtr->parentPtr));
+    //	    Tk_PathName(macWin->winPtr),
+    //	    Tk_PathName(macWin->winPtr->parentPtr));
 
     /*
      * Remove any dangling pointers that may exist if the window we are
@@ -472,7 +472,7 @@ XMoveResizeWindow(
 	    CGFloat YOff = (CGFloat) macWin->winPtr->wmInfoPtr->yInParent;
 	    NSRect r = NSMakeRect(
 		    X + XOff, TkMacOSXZeroScreenHeight() - Y - YOff - Height,
-	    	    Width, Height);
+		    Width, Height);
 
 	    [w setFrame:[w frameRectForContentRect:r] display:NO];
 	}
@@ -936,8 +936,7 @@ TkMacOSXVisableClipRgn(
 
 #if 0
 //This code is not currently used.  But it shows how to iterate over the
-//rectangles in a region described by an HIShape.  Probably we want to
-//replace the current dirtyRect by such a region.
+//rectangles in a region described by an HIShape.
 
 /*
  *----------------------------------------------------------------------
@@ -976,10 +975,6 @@ InvalViewRect(
 	break;
     case kHIShapeEnumerateRect:
 	dirtyRect = NSRectFromCGRect(CGRectApplyAffineTransform(*rect, t));
-	// Cannot rely on addTkDirtyRect: to force redrawing.
-	//MC This is the only place where the rect is not the view bounds.
-	//And it kills liveResize.
-	//[view generateExposeEvents:dirtyRect];
 	break;
     }
     [view generateExposeEvents:[view bounds]];
@@ -1023,7 +1018,7 @@ TkMacOSXRedrawViewIdleTask(
     TKContentView *view = (TKContentView *) clientData;
     //    fprintf(stderr, "idle redraw for %p\n", view);
     [view generateExposeEvents:[view bounds]];
-    [view setTkNeedsDisplay:YES];
+    [view setNeedsDisplay:YES];
 }
 
 void
@@ -1041,10 +1036,10 @@ TkMacOSXInvalidateWindow(
     Tk_Window parent = (Tk_Window) winPtr->parentPtr;
     TkMacOSXInvalClipRgns(tkwin);
     if ((flag == TK_PARENT_WINDOW) && parent){
-     	TkMacOSXInvalClipRgns(parent);
+	TkMacOSXInvalClipRgns(parent);
     }
     [view generateExposeEvents:[view bounds]];
-    [view setTkNeedsDisplay:YES];
+    [view setNeedsDisplay:YES];
 }
 
 /*
