@@ -132,51 +132,51 @@ namespace eval tk {
 
 	namespace export fixfocus
 	proc fixfocus {} {
-            catch {destroy .focus}
-            toplevel .focus
-            wm geometry .focus +0+0
-            entry .focus.e
-            .focus.e insert 0 "fixfocus"
-            pack .focus.e
-            update
-            focus -force .focus.e
-            destroy .focus
+	    catch {destroy .focus}
+	    toplevel .focus
+	    wm geometry .focus +0+0
+	    entry .focus.e
+	    .focus.e insert 0 "fixfocus"
+	    pack .focus.e
+	    update
+	    focus -force .focus.e
+	    destroy .focus
 	}
 
-        namespace export imageInit imageFinish imageCleanup imageNames
-        variable ImageNames
-        proc imageInit {} {
-            variable ImageNames
-            if {![info exists ImageNames]} {
-                set ImageNames [lsearch -all -inline -glob -not [lsort [image names]] ::tk::icons::indicator*]
-            }
-            imageCleanup
-            if {[lsort [image names]] ne $ImageNames} {
-                return -code error "IMAGE NAMES mismatch: [image names] != $ImageNames"
-            }
-        }
-        proc imageFinish {} {
-            variable ImageNames
+	namespace export imageInit imageFinish imageCleanup imageNames
+	variable ImageNames
+	proc imageInit {} {
+	    variable ImageNames
+	    if {![info exists ImageNames]} {
+		set ImageNames [lsearch -all -inline -glob -not [lsort [image names]] ::tk::icons::indicator*]
+	    }
+	    imageCleanup
+	    if {[lsort [image names]] ne $ImageNames} {
+		return -code error "IMAGE NAMES mismatch: [image names] != $ImageNames"
+	    }
+	}
+	proc imageFinish {} {
+	    variable ImageNames
 	    set imgs [lsearch -all -inline -glob -not [lsort [image names]] ::tk::icons::indicator*]
-            if {$imgs ne $ImageNames} {
-                return -code error "images remaining: [image names] != $ImageNames"
-            }
-            imageCleanup
-        }
-        proc imageCleanup {} {
-            variable ImageNames
-            foreach img [image names] {
-                if {$img ni $ImageNames} {image delete $img}
-            }
-        }
-        proc imageNames {} {
-            variable ImageNames
-            set r {}
-            foreach img [image names] {
-                if {$img ni $ImageNames} {lappend r $img}
-            }
-            return $r
-        }
+	    if {$imgs ne $ImageNames} {
+		return -code error "images remaining: [image names] != $ImageNames"
+	    }
+	    imageCleanup
+	}
+	proc imageCleanup {} {
+	    variable ImageNames
+	    foreach img [image names] {
+		if {$img ni $ImageNames} {image delete $img}
+	    }
+	}
+	proc imageNames {} {
+	    variable ImageNames
+	    set r {}
+	    foreach img [image names] {
+		if {$img ni $ImageNames} {lappend r $img}
+	    }
+	    return $r
+	}
 
 	#
 	#  CONTROL TIMING ASPECTS OF POINTER WARPING
@@ -241,6 +241,27 @@ namespace eval tk {
 	}
 	namespace export controlPointerWarpTiming
 
+	# On macOS windows are not allowed to overlap the menubar at the top of the
+	# screen or the dock.  So tests which move a window and then check whether it
+	# got moved to the requested location should use a y coordinate larger than the
+	# height of the menubar (normally 23 pixels) and an x coordinate larger than the
+	# width of the dock, if it happens to be on the left.
+	# menubarheight deals with this issue but may not be available from the test
+	# environment, therefore provide a fallback here
+	if {[llength [info procs menubarheight]] == 0} {
+	    if {[tk windowingsystem] ne "aqua"} {
+		# Windows may overlap the menubar
+		proc menubarheight {} {
+		    return 0
+		}
+	    } else {
+		# Windows may not overlap the menubar
+		proc menubarheight {} {
+		    return 30 ;  # arbitrary value known to be larger than the menubar height
+		}
+	    }
+	    namespace export menubarheight
+	}
     }
 }
 
@@ -272,7 +293,6 @@ testConstraint aquaKnownBug [expr {[testConstraint notAqua] || [testConstraint k
 
 # constraints for testing facilities defined in the tktest executable...
 testConstraint testImageType [expr {"test" in [image types]}]
-testConstraint testOldImageType [expr {"oldtest" in [image types]}]
 testConstraint testbitmap    [llength [info commands testbitmap]]
 testConstraint testborder    [llength [info commands testborder]]
 testConstraint testcbind     [llength [info commands testcbind]]
@@ -368,8 +388,8 @@ testConstraint secureserver 0
 if {[llength [info commands send]]} {
     testConstraint secureserver 1
     if {[catch {send $app set a 0} msg] == 1} {
-        if {[string match "X server insecure *" $msg]} {
-            testConstraint secureserver 0
+	if {[string match "X server insecure *" $msg]} {
+	    testConstraint secureserver 0
 	}
     }
 }
