@@ -4,12 +4,12 @@
 # tests.  Execute it by invoking "source all.tcl" when running tktest
 # in this directory.
 #
-# Copyright © 2007 by the Tk developers.
+# Copyright © 2007 the Tk developers.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 
-package require Tk ;# This is the Tk test suite; fail early if no Tk!
+package require tk ;# This is the Tk test suite; fail early if no Tk!
 package require tcltest 2.2
 tcltest::configure {*}$argv
 tcltest::configure -testdir [file normalize [file dirname [info script]]]
