@@ -4,7 +4,7 @@
  *	This file contains the platform specific routines for drawing 3d
  *	borders in the Motif style.
  *
- * Copyright (c) 1996 by Sun Microsystems, Inc.
+ * Copyright © 1996 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -13,7 +13,9 @@
 #include "tkInt.h"
 #include "tk3d.h"
 
-#if !(defined(_WIN32) || defined(MAC_OSX_TK))
+#if defined(MAC_OSX_TK)
+#include "tkMacOSXInt.h"
+#else
 #include "tkUnixInt.h"
 #endif
 
@@ -46,7 +48,7 @@ typedef struct {
 TkBorder *
 TkpGetBorder(void)
 {
-    UnixBorder *borderPtr = ckalloc(sizeof(UnixBorder));
+    UnixBorder *borderPtr = (UnixBorder *)ckalloc(sizeof(UnixBorder));
 
     borderPtr->solidGC = NULL;
     return (TkBorder *) borderPtr;
@@ -444,7 +446,7 @@ TkpGetShadows(
 
     if (borderPtr->shadow == None) {
 	borderPtr->shadow = Tk_GetBitmap(NULL, tkwin,
-		Tk_GetUid("gray50"));
+		"gray50");
 	if (borderPtr->shadow == None) {
 	    Tcl_Panic("TkpGetShadows couldn't allocate bitmap for border");
 	}
