@@ -4,7 +4,7 @@
  *	This module implements the Unix system defaults for the configuration
  *	package.
  *
- * Copyright (c) 1997 Sun Microsystems, Inc.
+ * Copyright © 1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -16,7 +16,7 @@
 /*
  *----------------------------------------------------------------------
  *
- * TkpGetSystemDefault --
+ * Tk_GetSystemDefault --
  *
  *	Given a dbName and className for a configuration option, return a
  *	string representation of the option.
@@ -33,11 +33,15 @@
  */
 
 Tcl_Obj *
-TkpGetSystemDefault(
+Tk_GetSystemDefault(
     Tk_Window tkwin,		/* A window to use. */
     const char *dbName,		/* The option database name. */
     const char *className)	/* The name of the option class. */
 {
+    (void)tkwin;
+    (void)dbName;
+    (void)className;
+
     return NULL;
 }
 
