@@ -5,7 +5,7 @@
  *	command, allowing commands to be passed from interpreter to
  *	interpreter.
  *
- * Copyright (C) 2002 Pat Thoyts <patthoyts@users.sourceforge.net>
+ * Copyright © 2002 Pat Thoyts <patthoyts@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -45,12 +45,12 @@ typedef struct {
  * TkWinSendCom public functions
  */
 
-HRESULT                 TkWinSendCom_CreateInstance(Tcl_Interp *interp,
-                            REFIID riid, void **ppv);
-int                     TkWinSend_QueueCommand(Tcl_Interp *interp,
-                            Tcl_Obj *cmdPtr);
-void                    SetExcepInfo(Tcl_Interp *interp,
-                            EXCEPINFO *pExcepInfo);
+MODULE_SCOPE HRESULT    TkWinSendCom_CreateInstance(Tcl_Interp *interp,
+	REFIID riid, void **ppv);
+MODULE_SCOPE int        TkWinSend_QueueCommand(Tcl_Interp *interp,
+	Tcl_Obj *cmdPtr);
+MODULE_SCOPE void       TkWinSend_SetExcepInfo(Tcl_Interp *interp,
+	EXCEPINFO *pExcepInfo);
 
 #endif /* _tkWinSendCom_h_INCLUDE */
 
